@@ -56,9 +56,9 @@ HafasMgateBackend::~HafasMgateBackend() = default;
 void HafasMgateBackend::init()
 {
     m_parser.setLocationIdentifierTypes(locationIdentifierType(), standardLocationIdentifierType());
-    m_parser.setLineModeMap(std::move(m_lineModeMap));
+    m_parser.setLineModeMap(m_lineModeMap);
     m_parser.setStandardLocationIdentfierCountries(std::move(m_uicCountryCodes));
-    m_parser.setPreferLineNumberProducts(std::move(m_lineNumberProducts));
+    m_parser.setProductNameMappings(std::move(m_productNameMappings));
 }
 
 AbstractBackend::Capabilities HafasMgateBackend::capabilities() const
@@ -151,6 +151,7 @@ bool HafasMgateBackend::queryJourney(const JourneyRequest &request, JourneyReply
                 jnyFltrL.push_back(jnyFltr);
             }
         }
+        addLineModeJourneyFilter(request.lineModes(), jnyFltrL);
         if (!jnyFltrL.isEmpty()) {
             req.insert(QLatin1String("jnyFltrL"),  jnyFltrL);
         }
@@ -223,6 +224,12 @@ bool HafasMgateBackend::queryStopover(const StopoverRequest &request, StopoverRe
         req.insert(QStringLiteral("time"), dt.toString(QStringLiteral("hhmmss")));
         req.insert(QStringLiteral("type"), request.mode() == StopoverRequest::QueryDeparture ? QLatin1String("DEP") : QLatin1String("ARR"));
 
+        QJsonArray jnyFltrL;
+        addLineModeJourneyFilter(request.lineModes(), jnyFltrL);
+        if (!jnyFltrL.isEmpty()) {
+            req.insert(QLatin1String("jnyFltrL"),  jnyFltrL);
+        }
+
         stationBoard.insert(QStringLiteral("meth"), QLatin1String("StationBoard"));
         stationBoard.insert(QStringLiteral("req"), req);
     }
@@ -408,14 +415,54 @@ void HafasMgateBackend::setConGroups(const QJsonArray &conGroups)
     }
 }
 
-void HafasMgateBackend::setPreferLineNumberProducts(const QJsonArray &lineNumberProduducts)
+static QStringList parseProductNameMappingFieldNames(const QJsonValue &val)
 {
-    m_lineNumberProducts.reserve(lineNumberProduducts.size());
-    for (const auto &lineNumV : lineNumberProduducts) {
-        const auto num = lineNumV.toInt();
-        if (num > 0) {
-            m_lineNumberProducts.push_back(num);
+    if (val.isString()) {
+        return QStringList({val.toString()});
+    }
+    if (val.isArray()) {
+        const auto a = val.toArray();
+        QStringList l;
+        l.reserve(a.size());
+        std::transform(a.begin(), a.end(), std::back_inserter(l), [](const auto &v) { return v.toString(); });
+        return l;
+    }
+    return {};
+}
+
+void HafasMgateBackend::setProductNameMappings(const QJsonArray &productNameMappings)
+{
+    m_productNameMappings.reserve(productNameMappings.size());
+    for (const auto &mV : productNameMappings) {
+        const auto mObj = mV.toObject();
+        HafasMgateProductNameMapping m;
+        m.cls = mObj.value(QLatin1String("cls")).toInt(-1);
+        m.lineName = parseProductNameMappingFieldNames(mObj.value(QLatin1String("lineName")));
+        m.routeName = parseProductNameMappingFieldNames(mObj.value(QLatin1String("routeName")));
+        m_productNameMappings.push_back(std::move(m));
+    }
+}
+
+void HafasMgateBackend::addLineModeJourneyFilter(const std::vector<Line::Mode> &lineModes, QJsonArray &jnyFltrL) const
+{
+    if (lineModes.empty()) {
+        return;
+    }
+
+    int productBitmask = 0;
+    for (const auto mode : lineModes) {
+        for (const auto& [key, value] : m_lineModeMap) {
+            if (value == mode)  {
+                productBitmask |= key;
+            }
         }
     }
-    std::sort(m_lineNumberProducts.begin(), m_lineNumberProducts.end());
+
+    if (productBitmask != 0) {
+        jnyFltrL.push_back(QJsonObject({
+            {QLatin1String("type"), QLatin1String("PROD")},
+            {QLatin1String("mode"), QLatin1String("INC")},
+            {QLatin1String("value"), QString::number(productBitmask)}
+        }));
+    }
 }
