@@ -5,11 +5,13 @@
 */
 
 #include "hafasmgateparser.h"
+#include "hafasconfiguration.h"
 #include "hafasvehiclelayoutparser.h"
 #include "logging.h"
 #include "datatypes/loadutil_p.h"
 #include "geo/polylinedecoder_p.h"
 #include "ifopt/ifoptutil.h"
+#include "json/jsonpointer_p.h"
 
 #include <KPublicTransport/Journey>
 #include <KPublicTransport/Platform>
@@ -255,24 +257,33 @@ std::vector<Location> HafasMgateParser::parseLocations(const QJsonArray &locL) c
     return locs;
 }
 
-std::vector<Line> HafasMgateParser::parseLines(const QJsonArray &prodL, const std::vector<Ico> &icos) const
+std::vector<Route> HafasMgateParser::parseProducts(const QJsonArray &prodL, const std::vector<Ico> &icos) const
 {
-    std::vector<Line> lines;
-    lines.reserve(prodL.size());
+    std::vector<Route> routes;
+    routes.reserve(prodL.size());
     for (const auto &prodV : prodL) {
         const auto prodObj = prodV.toObject();
         const auto prodCls = prodObj.value(QLatin1String("cls")).toInt();
 
+        Route route;
         Line line;
         line.setMode(parseLineMode(prodCls));
 
-        if (std::binary_search(m_lineNumberProducts.begin(), m_lineNumberProducts.end(), prodCls)) {
-            line.setName(prodObj.value(QLatin1String("line")).toString());
-            if (line.name().isEmpty()) {
-                line.setName(prodObj.value(QLatin1String("nameS")).toString());
+        const auto it = std::find(m_productNameMappings.begin(), m_productNameMappings.end(), prodCls);
+        if (it != m_productNameMappings.end()) {
+            for (const auto &lineName : (*it).lineName) {
+                line.setName(JsonPointer::evaluate(prodObj, lineName).toString());
+                if (!line.name().isEmpty()) {
+                    break;
+                }
             }
-        }
-        if (line.name().isEmpty()) {
+            for (const auto &routeName : (*it).routeName) {
+                route.setName(JsonPointer::evaluate(prodObj, routeName).toString());
+                if (!route.name().isEmpty()) {
+                    break;
+                }
+            }
+        } else {
             line.setName(prodObj.value(QLatin1String("name")).toString());
         }
 
@@ -282,10 +293,11 @@ std::vector<Line> HafasMgateParser::parseLines(const QJsonArray &prodL, const st
             line.setTextColor(icos[icoIdx].fg);
         }
 
-        lines.push_back(line);
+        route.setLine(std::move(line));
+        routes.push_back(std::move(route));
     }
 
-    return lines;
+    return routes;
 }
 
 static QString parsePlatform(const QJsonObject &obj, char ad, char rs)
@@ -304,7 +316,7 @@ std::vector<Stopover> HafasMgateParser::parseStationBoardResponse(const QJsonObj
     const auto commonObj = obj.value(QLatin1String("common")).toObject();
     const auto icos = parseIcos(commonObj.value(QLatin1String("icoL")).toArray());
     const auto locs = parseLocations(commonObj.value(QLatin1String("locL")).toArray());
-    const auto lines = parseLines(commonObj.value(QLatin1String("prodL")).toArray(), icos);
+    const auto products = parseProducts(commonObj.value(QLatin1String("prodL")).toArray(), icos);
     const auto remarks = parseRemarks(commonObj.value(QLatin1String("remL")).toArray());
     const auto warnings = parseWarnings(commonObj.value(QLatin1String("himL")).toArray());
 
@@ -318,11 +330,11 @@ std::vector<Stopover> HafasMgateParser::parseStationBoardResponse(const QJsonObj
 
         Stopover dep;
         Route route;
-        route.setDirection(jnyObj.value(QLatin1String("dirTxt")).toString());
-        const auto lineIdx = jnyObj.value(QLatin1String("prodX")).toInt(-1);
-        if (lineIdx >= 0 && (unsigned int)lineIdx < lines.size()) {
-            route.setLine(lines[lineIdx]);
+        const auto prodIdx = jnyObj.value(QLatin1String("prodX")).toInt(-1);
+        if (prodIdx >= 0 && (unsigned int)prodIdx < products.size()) {
+            route = products[prodIdx];
         }
+        route.setDirection(jnyObj.value(QLatin1String("dirTxt")).toString());
 
         const auto dateStr = jnyObj.value(QLatin1String("date")).toString();
         dep.setScheduledDepartureTime(parseDateTime(dateStr, stbStop.value(QLatin1String("dTimeS")), stbStop.value(QLatin1String("dTZOffset"))));
@@ -627,7 +639,7 @@ std::vector<Journey> HafasMgateParser::parseTripSearch(const QJsonObject &obj)
     const auto commonObj = obj.value(QLatin1String("common")).toObject();
     const auto icos = parseIcos(commonObj.value(QLatin1String("icoL")).toArray());
     const auto locs = parseLocations(commonObj.value(QLatin1String("locL")).toArray());
-    const auto lines = parseLines(commonObj.value(QLatin1String("prodL")).toArray(), icos);
+    const auto products = parseProducts(commonObj.value(QLatin1String("prodL")).toArray(), icos);
     const auto remarks = parseRemarks(commonObj.value(QLatin1String("remL")).toArray());
     const auto warnings = parseWarnings(commonObj.value(QLatin1String("himL")).toArray());
     const auto loadInfos = parseLoadInformation(commonObj.value(QLatin1String("tcocL")).toArray());
@@ -689,11 +701,11 @@ std::vector<Journey> HafasMgateParser::parseTripSearch(const QJsonObject &obj)
 
                 const auto jnyObj = secObj.value(QLatin1String("jny")).toObject();
                 Route route;
-                route.setDirection(jnyObj.value(QLatin1String("dirTxt")).toString());
-                const auto lineIdx = jnyObj.value(QLatin1String("prodX")).toInt();
-                if ((unsigned int)lineIdx < lines.size()) {
-                    route.setLine(lines[lineIdx]);
+                const auto prodIdx = jnyObj.value(QLatin1String("prodX")).toInt(-1);
+                if (prodIdx >= 0 && (unsigned int)prodIdx < products.size()) {
+                    route = products[prodIdx];
                 }
+                route.setDirection(jnyObj.value(QLatin1String("dirTxt")).toString());
                 section.setRoute(route);
 
                 if (jnyObj.value(QLatin1String("isCncl")).toBool()) {
@@ -836,7 +848,7 @@ QDateTime HafasMgateParser::parseDateTime(const QString &date, const QJsonValue
     return dt;
 }
 
-void HafasMgateParser::setPreferLineNumberProducts(std::vector<int> &&lineNumberProducts)
+void HafasMgateParser::setProductNameMappings(std::vector<HafasMgateProductNameMapping> &&productNameMappings)
 {
-    m_lineNumberProducts = std::move(lineNumberProducts);
+    m_productNameMappings = std::move(productNameMappings);
 }
