@@ -9,6 +9,7 @@
 
 #include "kpublictransport_export.h"
 #include "hafasparser.h"
+#include "hafasconfiguration.h"
 
 #include <KPublicTransport/Disruption>
 #include <KPublicTransport/Line>
@@ -41,7 +42,7 @@ class KPUBLICTRANSPORT_EXPORT HafasMgateParser : public HafasParser
 public:
     HafasMgateParser();
     ~HafasMgateParser();
-    void setPreferLineNumberProducts(std::vector<int> &&lineNumberProducts);
+    void setProductNameMappings(std::vector<HafasMgateProductNameMapping> &&productNameMappings);
 
     std::vector<Stopover> parseDepartures(const QByteArray &data) const;
     std::vector<Location> parseLocations(const QByteArray &data) const;
@@ -54,12 +55,12 @@ public:
 private:
     Q_DISABLE_COPY(HafasMgateParser)
     std::vector<Stopover> parseStationBoardResponse(const QJsonObject &obj) const;
-    std::vector<Line> parseLines(const QJsonArray &prodL, const std::vector<Ico> &icos) const;
+    std::vector<Route> parseProducts(const QJsonArray &prodL, const std::vector<Ico> &icos) const;
     std::vector<Location> parseLocations(const QJsonArray &locL) const;
     std::vector<Journey> parseTripSearch(const QJsonObject &obj);
     bool parseError(const QJsonObject &obj) const;
 
-    std::vector<int> m_lineNumberProducts;
+    std::vector<HafasMgateProductNameMapping> m_productNameMappings;
 };
 
 }
