@@ -68,7 +68,8 @@ function parseZugportalTrip(response)
                 mode = response.type == 'CITY_TRAIN' ? 'RapidTransit' : 'Train'
             }
         },
-        intermediateStops = []
+        intermediateStops = [],
+        notes = []
     };
     for (s of response.stops) {
         let stop = {
@@ -78,8 +79,13 @@ function parseZugportalTrip(response)
                 identifier = { ibnr = s.station.evaNo }
             },
             scheduledPlatform = s.track.target,
-            expectedPlatform = s.track.prediction
+            expectedPlatform = s.track.prediction,
+            notes = []
         };
+        if (s.station.position) {
+            stop.stopPoint.latitude = s.station.position.latitude;
+            stop.stopPoint.longitude = s.station.position.longitude;
+        }
         if (s.departureTime) {
             stop.scheduledDepartureTime = s.departureTime.target;
             stop.expectedDepartureTime = s.departureTime.predicted;
@@ -88,9 +94,24 @@ function parseZugportalTrip(response)
             stop.scheduledArrivalTime = s.arrivalTime.target;
             stop.expectedArrivalTime = s.arrivalTime.predicted;
         }
-        // TODO messages [] - no example whats in there yet, status != "Normal"?
+
+        if (s.status === "Canceled" || s.canceled === true) {
+            stop.disruptionEffect = 'NoService';
+        }
+
+        for (message of s.messages) {
+            if (message.type === "CUSTOMER_REASON") {
+                notes.push(message.textShort || message.text);
+            }
+        }
+
         section.intermediateStops.push(stop);
     }
-    // TODO uic, trainNo, hims[]
+
+    for (him of response.hims) {
+        section.notes.push(him.captionHtml || him.caption);
+    }
+
+    // TODO uic, trainNo
     return { sections = [section] };
 }
